unit UBoss;

interface

uses
  DXSprite, DXInput, UVector, UConsts, Types, Math, SysUtils,
  UBullet, UScore, MPlayer;

type
  TEnemySprite = class(TImageSprite)
  private
    FVector : TVector2D;
    FShaked : Boolean;
    FAmplitude : integer;
    //FLines : array of TLine;

    ///procedure Intersect();
    procedure Shake();
  protected

    procedure DoMove(MoveCount: Integer); override;
    procedure DoCollision(Sprite:TSprite; var Done:Boolean); override;

    //procedure SetLines(NewLines : array of TLine);

  public

    constructor Create(Parent : TSprite); override;
    destructor Destroy(); override;

    property Vector : TVector2D read FVector write FVector;
    property Shaked : Boolean read FShaked write FShaked;
    property Amplitude : integer read FAmplitude write FAmplitude;
  end;

  TBossSprite = class(TEnemySprite)
  private
    procedure Jump(const JumpHeight : Double; Kind : Byte);
  end;

  TBowserSprite = class(TBossSprite)
  private
    DropPos : Byte;
  protected
    procedure DoMove(MoveCount: Integer); override;
  public
    constructor Create(Parent : TSprite); override;
  end;

  TPokeySprite = class(TBossSprite)
  private
    FNumber : Byte;
    FActionNumber : Word;
    Hid : Boolean;
    BTick : integer;

    function Aim(const AimX, AimY : double) : Boolean;//True
    procedure ChangeEventNumber();
    procedure MakeBlank();
  protected
    procedure DoMove(MoveCount: Integer); override;
  public
    constructor Create(Parent : TSprite); override;
    procedure Reset(const NewNumber : Byte);

    property Number : Byte read FNumber;
    property ActionNumber : Word read FActionNumber;
  end;

var
  RealNumber : Byte;

implementation

uses
  Shadow, DXDraws,UHero;

constructor TEnemySprite.Create(Parent : TSprite);
begin
  inherited;
  FVector := CreateVector2D(0,0);
  Amplitude := 5;
  Image := Form1.DXImageList1.Items.Find('Boss');
  with Image do
  begin
    TransparentColor := Picture.BitMap.canvas.pixels[0,0];
    ThemeColor := TransparentColor;
    PatternWidth := Picture.Width div (AnimCount + 1);
    Restore;
  end;
  Width := Image.Width;
  Height := Image.Height;
  Collisioned := True;
  PixelCheck := True;
  Shaked := False;
end;

destructor TEnemySprite.Destroy();
begin
  inherited;
end;

procedure TEnemySprite.DoMove(MoveCount: Integer);
begin
  inherited;
  if Shaked then Shake();
  X := X + Vector.X;
  Y := Y + Vector.Y;
  Collision();
end;

var
  Theta : integer;
procedure TEnemySprite.Shake();
var
  NewSin,NewCos : Extended;
begin
  Inc(Theta);
  Dec(FAmplitude);
  SinCos(Theta, NewSin, NewCos);
  X := X + NewCos * (Amplitude * 0.3);
  Y := Y + NewSin * (Amplitude * 0.3);
  if Amplitude < 1 then Shaked := False;
end;

var
  NoteTick : integer;
procedure TEnemySprite.DoCollision(Sprite:TSprite; var Done:Boolean);
begin
  inherited;
  if Sprite is THeroSprite then
  begin
    with Hero do
    begin
      if DashType = None then
      begin
        if (Hero.IntactTime = 0) and
           (not Shaked) then
        begin
          ComboCount := 0;
          HP := HP - ENEMY_AP;
          IntactTime := ENEMY_INTACT_TIME;
        end;
      end
        else
      begin
        if (Self is TPokeySprite) and
           (TPokeySprite(Self).Number <> RealNumber) then
        begin
          with TPokeySprite(Self) do
          begin
            if not Hid then
            begin
              Inc(BTick);
              if BTick mod 5 = 1 then Form1.DXWaveList1.Items.Find('Bass').Play(False);
              if BTick > 40 then
              begin
                Collisioned := false;
                AnimPos := 1;
              end;
            end;
          end;
        end
          else
        begin
          Inc(NoteTick);
          if (NoteTick mod 3 = 0) then
          begin
            Hero.ComboCount := Hero.ComboCount + 1;
            with TRuinSprite.Create(Form1.DXSpriteEngine1.Engine) do
            begin
              MakeNote(TRuinType(Random(4) + 2));
              Vector.X := NormalizeVector2D(Hero.Vector).X * 3;
              Vector.Y := -Random(25);
              X := Hero.X;
              Y := Hero.Y;
            end;
          end;
        end;
      end;
    end;
  end;
end;

procedure TBossSprite.Jump(const JumpHeight : Double; Kind : Byte);
var
  LInterval, NInterval : Integer;
  NextEv, LastEv : PEvent;
begin
  NextEv := Score.FindEvent(False,Kind);
  LastEv := Score.FindEvent(True,Kind);
  if (NextEv <> nil) and
     (LastEv <> nil) and
     (NextEv <> LastEv) then
  begin
    //ǂ琻쎞Ԃ30%͂7sɔ₳ꂽ݂ł
    LInterval := Score.NowPosition - LastEv.Timing;
    NInterval := NextEv.Timing - Score.NowPosition;
    Y :=Form1.ClientHeight - Height - EARTH_POS
         +( Sqr(Sqrt(LInterval + NInterval) *
                 (2 * LInterval / (LInterval + NInterval) - 1)
               ) - LInterval - NInterval
          ) * JumpHeight;
  end
    else
  begin
    Y := Form1.ClientHeight - Height - EARTH_POS ;
  end;
  if Y > Form1.ClientHeight - Height - EARTH_POS  then
  begin
    Y := Form1.ClientHeight - Height - EARTH_POS ;
  end;
end;

(***************************
        TBowserSprite
****************************)
constructor TBowserSprite.Create(Parent : TSprite);
begin
  AnimCount := 1;
  inherited;

  X := 100;
  Y := 0;
  FVector.X := BOWSER_SPEED;
end;

procedure TBowserSprite.DoMove(MoveCount: integer);
begin
//  FVector.X := Random(5) - 2;
  inherited;
  if not Shaked then
  begin
    if Vector.X <= 0 then
    begin
      AnimPos := 1;
      if X < 0 then
      begin
        X := 0;
        FVector.X := BOWSER_SPEED;
      end;
    end
      else
    begin
      AnimPos := 0;
      if X > Form1.ClientWidth - Width then
      begin
        X := Form1.ClientWidth - Width;
        FVector.X := -BOWSER_SPEED;
      end;
    end;
    Jump(0.2,2);
    if (Score.Passed) and (Score.NextEvent <> Score.LastEvent) then
    begin
      case Score.LastEvent.Kind of
        2 : Form1.DXWaveList1.Items.Find('Bass').Play(False);
        1 : begin //
          Inc(DropPos);
          DropPos := DropPos mod (Form1.ClientWidth div 100);
          TBulletSprite.Create(Form1.DXSpriteEngine1.Engine).
            Reset(1, DropPos * 100, 0, CreateVector2D(0, -6));
          Form1.DXWaveList1.Items.Find('Snare').Play(False);
        end;
        3 : begin //fJC
          TBulletSprite.Create(Form1.DXSpriteEngine1.Engine).
            Reset(2, 50, -350, CreateVector2D(0, 0));
          Form1.DXWaveList1.Items.Find('Snare').Play(False);
        end;
        4 : begin//[炭
          Inc(DropPos);
          DropPos := DropPos mod (Form1.ClientWidth div 100);
          TBulletSprite.Create(Form1.DXSpriteEngine1.Engine).
            Reset(1, DropPos * 100, 0, CreateVector2D(0, -6));
          TBulletSprite.Create(Form1.DXSpriteEngine1.Engine).
            Reset(1, Form1.ClientWidth - DropPos * 100 - 100, 0,CreateVector2D(0, -6));
          Form1.DXWaveList1.Items.Find('Snare').Play(False);
        end;
      end;
    end;
  end;
  if Score.LastEvent <> nil then
  begin
    if Score.LastEvent.Kind = 8 then
    begin
      Shaked := True;
      FVector := CreateVector2D(0,0);
      Amplitude := Amplitude + 5;
      if Form1.MediaPlayer1.Mode <> mpPlaying then
      begin
        Dead();
      end;
    end;
  end;
end;



(***************************
        TPokeySprite
****************************)
constructor TPokeySprite.Create(Parent : TSprite);
begin
  AnimCount := 1;
  inherited;
  X := (Form1.ClientWidth - Width) div 2;
  Y := - Image.Height;
  FActionNumber := 0;
  PixelCheck := false;
end;

procedure TPokeySprite.Reset(const NewNumber : Byte);
begin
  FNumber := NewNumber;
  Collisioned := True;
  AnimPos := 0;
  PixelCheck := RealNumber = Number;
end;

var
  //(]鎞炢oԂ̂Ȃl)
  CenterX, CenterY :Double;
  Angle : Double;
procedure TPokeySprite.DoMove(MoveCount: Integer);
var
  NewSin, NewCos : Extended;
begin
  inherited;
  if not Collisioned then AnimPos := 1;
  Hid := false;
  if Hero <> nil then
  begin
    if (Collisioned) and (Ready) and (Hero.IntactTime <= 0) and
       (Scene = BattleScene) and (not Shaked) then Hero.HP := Hero.HP - 1.5;
  end;
  case ActionNumber of
    0 : begin
      ChangeEventNumber(); //܂ł܂āI
      MakeBlank();
    end;
    1 : begin    //ɂĂ
      if Aim(X, Form1.ClientHeight - EARTH_POS - Height) then
      begin
        Inc(FActionNumber);
        FVector.X := 0;
        FVector.Y := 0;
      end;
      MakeBlank();
    end;
    2 : begin   //L
      if Aim((Form1.ClientWidth - Width) div 2 + (Number - 2) * 130 , Y) then
      begin
        FVector.Y := 0;
      end;
      if (Score.Passed) and (Score.LastEvent.Kind = 2) then
      begin
        Inc(FActionNumber);
        case Number of
          1 : FVector.X := 2;
          2 : FVector.X := 3;
          3 : FVector.X := 4;
        end;
      end;
    end;
    3 : begin
      if Score.NextEvent.Kind = 2 then
      begin
        //Wv܂
        if (Vector.X < 0) and (X < 0) then
        begin
          X := 0;
          FVector.X := -Vector.X;
        end
          else if (Vector.X > 0) and (X > Form1.ClientWidth - Width) then
        begin
          X := Form1.ClientWidth - Width;
          FVector.X := -Vector.X;
        end;
        Jump(0.1 * Number * 1.5,2);
      end
        else
      begin
        //ŏIIɂ͈
        if Aim((Form1.ClientWidth - Width) div 2 + (Number - 2) * 130 , Y) then
        begin
          FVector.X := 0;
          FVector.Y := 0;
          Inc(FActionNumber);
        end;
      end;
    end;
    4, 11 : begin
      if Score.NextEvent.Kind <> 3 then
      begin
        if Aim((Form1.ClientWidth - Width) div 2,
               (Form1.ClientHeight - Height) div 2) then
        begin
          Inc(FActionNumber);
          MakeBlank();
        end;
      end
        else
      begin
        //݂ȂłăWv
        Jump(0.15, 3);
        if Score.Passed then
        begin
          if Vector.X = 0 then FVector.X := POKEY_SPEED div 2
            else FVector.X := -Vector.X;
        end;
      end;
    end;
    5, 12 : begin   //܂҂
      ChangeEventNumber();
      MakeBlank();
    end;
    6, 13 : begin
      //Ⴎ
      if GetVector2DLength(Vector) = 0 then
      begin
        case Number of
          1 : FVector := CreateVector2D(5, 2);
          2 : FVector := CreateVector2D(-2, 5);
          3 : FVector := CreateVector2D(1, -2);
        end;
      end;
      if (Vector.X < 0) and (X < 0) then
      begin
        X := 0;
        FVector.X := -Vector.X;
      end
        else if (Vector.X > 0) and (X > Form1.ClientWidth - Width) then
      begin
        X := Form1.ClientWidth - Width;
        FVector.X := -Vector.X;
      end;
      if (Vector.Y < 0) and (Y < 0) then
      begin
        Y := 0;
        FVector.Y := -Vector.Y;
      end
        else if (Vector.Y > 0) and (Y > Form1.ClientHeight - Height - EARTH_POS) then
      begin
        Y := Form1.ClientHeight - Height - EARTH_POS;
        FVector.Y := -Vector.Y;
      end;

      ChangeEventNumber();
    end;
    7, 14, 17: begin
      //܂W
      if Aim((Form1.ClientWidth - Width) div 2,
             (Form1.ClientHeight - Height) div 2) then MakeBlank;
      if Score.Passed then Inc(FActionNumber);
    end;
    8 , 15, 18: begin //]Oɔzuɂ
      if Score.LastEvent.Kind - 3 = Number then
      begin
        case Number of
          1:Aim((Form1.ClientWidth - Width) div 2 + 0.86 * 100,
                (Form1.ClientHeight - Height) div 2 + 0.5 * 100);
          2:Aim((Form1.ClientWidth - Width) div 2 - 0.86 * 100,
                (Form1.ClientHeight - Height) div 2 + 0.5 * 100);
          3:Aim((Form1.ClientWidth - Width) div 2,
                (Form1.ClientHeight - Height) div 2 - 100);
        end;
      end;
      if (Score.Passed) and (Score.LastEvent.Kind = 7) then
      begin
        Inc(FActionNumber);
        if Number = 3 then
        begin
          CenterX := X;
          CenterY := Y + 50;
        end;
        Angle := 0;
      end;
    end;
    9 , 16, 19: begin //邮邮邮
      SinCos(Angle + Pi / 12 + Pi / 3 * 2 * (Number - 1), NewSin, NewCos);
      X := CenterX + NewCos * 100;
      Y := CenterY + NewSin * 100;
      Angle := Angle + 0.04;
      if Hero <> nil then
      begin
        SinCos(ArcTan2(Hero.Y - CenterY - 200, Hero.X - CenterX), NewSin, NewCos);
        CenterX := CenterX + NewCos * 1.5;
        CenterY := CenterY + NewSin * 1.5;
      end;
      ChangeEventNumber();
    end;
    10, 20 : begin  //܂ɂȂ
      if Aim((Form1.ClientWidth - Width) div 2 + (Number - 2) * 130 , Y) then
      begin
        FVector.X := 0;
        FVector.Y := 0;
        Inc(FActionNumber);
      end;
    end;
    //114ƓB
    //12
    //13141516
    //17ł܂Ƃ܂18ł܂܂͂߂19ł܂20ł܂Ă
  end;
  X := X + Vector.X;
  Y := Y + Vector.Y;

  //ՏI
  if Score.LastEvent <> nil then
  begin
    if Score.LastEvent.Kind = 8 then
    begin
      Shaked := True;
      FVector := CreateVector2D(0,0);
      Amplitude := Amplitude + 5;
      if Form1.MediaPlayer1.Mode <> mpPlaying then
      begin
        Dead();
      end;
    end;
  end;

end;

procedure TPokeySprite.ChangeEventNumber();
begin
  if (Score.Passed) and (Score.LastEvent.Kind = 7) then
  begin
    Inc(FActionNumber);
  end;
end;

function TPokeySprite.Aim(const AimX, AimY : double) : Boolean;//True
var
  AimWay : TVector2D;
begin
  AimWay := CreateVector2D(AimX - X ,AimY - Y);
  if GetVector2DLength(AimWay) <= POKEY_SPEED * 2 then
  begin
    X := AimX;
    Y := AimY;
    FVector.X := 0;
    FVector.Y := 0;
    Result := True;
  end
    else
  begin
    SetVector2DLength(AimWay, POKEY_SPEED);
    FVector := AimWay;
    Result := False;
  end;
end;

procedure TPokeySprite.MakeBlank();
begin
  AnimPos := 0;
  Collisioned := True;
  Hid := True;
  BTick := 0;
end;

end.
