unit UScore;

interface

uses
  Classes, SysUtils;


type
  PEvent = ^TEvent;
  TEvent = record
    Timing : integer;
    Kind : Byte;
  end;

  TScore = class(TList)
  private
    FNextEventNumber : Word;
    FNextEvent : PEvent;
    FLastEventNumber : Word;
    FLastEvent : PEvent;
    FPassed : Boolean;
    FNearestEventNumber : Word;
    FNearestEvent : PEvent;
    FNearestEventInterval : integer;
    FNowPosition : integer;
    FInterval : integer;

    function Get(Index: Integer): PEvent;
    procedure Put(Index: Integer; const Value: PEvent);
  public
    constructor Create();
    destructor Destroy(); override;

    procedure Initialize();
    procedure Load();
    procedure Scan();
    procedure DisposeItems();
    function FindEvent(const Pre:Boolean; Kind:Integer) : PEvent;

    property Items[Index: Integer]: PEvent read Get write Put; default;
    property NextEventNumber : Word read FNextEventNumber;
    property NextEvent : PEvent read FNextEvent;
    property LastEventNumber : Word read FLastEventNumber;
    property LastEvent : PEvent read FLastEvent;
    property Passed : Boolean read FPassed;
    property NearestEventNumber : Word read FNearestEventNumber;
    property NearestEvent : PEvent read FNearestEvent;
    property NearestEventInterval : integer read FNearestEventInterval;
    property NowPosition : integer read FNowPosition;
    property Interval : integer read FInterval;
  end;

  function SortComp(Item1, Item2: Pointer): Integer;

var
  Score : TScore;

implementation
uses
  Shadow;

constructor TScore.Create();
begin
  inherited ;
  Initialize();
end;

destructor TScore.Destroy();
begin
  DisposeItems();
  inherited;
end;

procedure TScore.Initialize();
begin
  FNextEventNumber := 0;
  FNextEvent := nil;
  FLastEventNumber := 0;
  FLastEvent := nil;
  FNearestEventNumber := 0;
  FNearestEvent := nil;
  FNearestEventInterval := 0;
  FNowPosition := 0;
  FInterval := 0;
end;

procedure TScore.Load();
var
  Sco1,ExSco:TStrings;
  i:integer;
  NewEvent : PEvent;
begin
  DisposeItems();
  Sco1:=TStringList.Create;
  ExSco:=TStringList.Create;
  try
    Sco1.LoadFromFile(ExePath + 'Score\' +  Stages[StageNumber].MusicTitle + '.txt');
    ExSco.Assign(Sco1);
    Capacity := ExSco.Count + 1;
    New(NewEvent);
    NewEvent.Timing := 0;
    NewEvent.Kind := 1;
    Add(NewEvent);
    for i:= 0 to ExSco.Count - 1 do
    begin
      try
        Sco1.CommaText:=ExSco[i];
        New(NewEvent);
        NewEvent.Timing := StrToIntDef(Sco1[0],1000);
        NewEvent.Kind := StrToIntDef(Sco1[1],0);
        Add(NewEvent);
      except
        Form1.Close();
      end;
    end;
  except
    FreeAndNil(Sco1);
    FreeAndNil(ExSco);
  end;
  Sort(SortComp);
  Initialize();
end;

procedure TScore.Scan();
var
  i : integer;
  lei : integer;
begin
  FNowPosition := Form1.MediaPlayer1.Position;
  FPassed := False;
  for i := LastEventNumber + 1 to Count - 1 do
  begin
    if Items[i].Timing <= NowPosition then
    begin
      FLastEventNumber := i;
      if i = Count - 1 then
      begin
        FNextEventNumber := i;
      end
        else
      begin
        FNextEventNumber := i + 1;
      end;
      FPassed := True;
    end;
  end;
  FNextEvent := Items[NextEventNumber];
  FLastEvent := Items[LastEventNumber];
  FInterval := NowPosition - NextEvent.Timing;
  lei := NowPosition - LastEvent.Timing;
  if abs(Interval) <= abs(lei) then
  begin
    FNearestEventInterval := Interval;
    FNearestEventNumber := NextEventNumber;
  end
    else
  begin
    FNearestEventInterval := lei;
    FNearestEventNumber := LastEventNumber;
  end;

  FNearestEvent := Items[NearestEventNumber];

end;

function TScore.FindEvent(const Pre:Boolean; Kind:Integer) : PEvent;
var
  i:integer;
begin
  Result := nil;
  if Pre then
  begin
    for i:= LastEventNumber downto 0 do
    begin
      if Items[i].Kind = Kind then
      begin
        Result := Items[i];
        Exit;
      end;
    end;
  end
    else
  begin
    for i:= NextEventNumber to Count - 1 do
    begin
      if Items[i].Kind = Kind then
      begin
        Result := Items[i];
        Exit;
      end;
    end
  end;
end;

procedure TScore.DisposeItems();
var
  i : integer;
begin
  for i := 0 to Count - 1 do
  begin
    Dispose(List[i]);
  end;
  Clear();
end;

function TScore.Get(Index: Integer): PEvent;
begin
  Result :=  PEvent( inherited Get(Index) );
end;

procedure TScore.Put(Index: Integer; const Value: PEvent);
begin
  inherited Put( Index, Value );
end;

function SortComp(Item1, Item2: Pointer): Integer;
begin
  Result := PEvent(Item1).Timing - PEvent(Item2).Timing;
end;

end.
