unit UVector;

interface

uses
  Math;

type
  TVector2D = record
    X,Y : Double;
  end;

  function CreateVector2D(const X,Y : Double) : TVector2D;

  function NormalizeVector2D(Vector2D : TVector2D) : TVector2D;
  function ScaleVector2D(const Vector2D : TVector2D; Scale:Double) : TVector2D;
  procedure SetVector2DLength(var Vector2D : TVector2D; const Scale:Double);
  function GetVector2DLength(const Vector2D : TVector2D) : Double;

implementation

function CreateVector2D(const X,Y : Double) : TVector2D;
var
  Vector2D:TVector2D;
begin
  Vector2D.X := X;
  Vector2D.Y := Y;
  Result := Vector2D;
end;

function NormalizeVector2D(Vector2D : TVector2D) : TVector2D;
var
  Length : Double;
begin
  Length := GetVector2DLength(Vector2D);
  if Length > 0 then Length := 1 / Length;
  Result := CreateVector2D(Vector2D.X * Length,
                           Vector2D.Y * Length);
end;

function ScaleVector2D(const Vector2D : TVector2D; Scale:Double) : TVector2D;
var
  Vec2D : TVector2D;
begin
  Vec2D.X := Vector2D.X * Scale;
  Vec2D.Y := Vector2D.Y * Scale;
  Result := Vec2D;
end;

procedure SetVector2DLength(var Vector2D : TVector2D; const Scale:Double);
begin
  Vector2D := ScaleVector2D(NormalizeVector2D(Vector2D) , Scale);
end;

function GetVector2DLength(const Vector2D : TVector2D) : Double;
begin
  Result := Sqrt(Sqr(Vector2D.X) + Sqr(Vector2D.Y));
end;

end.
